/**
 *   common-jbi-runtime.jar - Set of classes used in building JBI Components
 *
 *   Copyright (C) 2007 Gestalt, LLC. All Rights Reserved.
 *   http://www.gestalt-llc.com/
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License version 2.1 as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 */

package com.gestalt.jbi.component.lifecycle;

import com.gestalt.jbi.component.AbstractComponent;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.messaging.DeliveryChannel;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author rriven
 *
 */
public abstract class AbstractComponentLifeCycle implements ComponentLifeCycle {

	protected static final Logger log = Logger.getLogger(AbstractComponentLifeCycle.class.getName());
	
	protected AbstractComponent component;
	
	protected ComponentContext componentContext;
	
	protected DeliveryChannel deliveryChannel;
	
	protected ObjectName mBeanName;
	
	protected MBeanServer mBeanServer;
	
	protected Object mBean;
	
	protected AbstractMessageExchangeProcessor processor;
	
	public AbstractComponentLifeCycle(AbstractComponent component) {
		this.component = component;		
	}
	
	/* (non-Javadoc)
	 * @see javax.jbi.component.ComponentLifeCycle#getExtensionMBeanName()
	 */
	public ObjectName getExtensionMBeanName() {
		return mBeanName;
	}
	
	public ComponentContext getComponentContext() {
		return componentContext;
	}

	/* (non-Javadoc)
	 * @see javax.jbi.component.ComponentLifeCycle#init(javax.jbi.component.ComponentContext)
	 */
	public void init(ComponentContext componentContext) throws JBIException {		
		this.componentContext = componentContext;
		log.log(Level.INFO, "Initializing abstract component lifecycle");
		mBeanName = createMBeanName();
		mBeanServer = componentContext.getMBeanServer();
		processor = createMessageExchangeProcessor();
		processor.init();
		doInit();
		if (mBeanName == null) {
			return ;
		}
		if (mBeanServer == null) {
			log.warning("A null MBeanServer was detected.");
			return ;
		}
		mBean = createMBean();
		if (mBean == null) {
			log.warning("A null MBean was detected");
			return ;
		}
		try {
			if (mBeanServer.isRegistered(mBeanName)) {
				mBeanServer.unregisterMBean(mBeanName);
			}
			mBeanServer.registerMBean(mBean, mBeanName);
		} catch (Exception e) {
			log.throwing(e.getMessage(), null, e);
			throw new JBIException("Error managing lifecycle MBean: " + e.getMessage(), e);
		}
	}
	
	public abstract Object createMBean();
	
	public abstract ObjectName createMBeanName();
	
	public abstract AbstractMessageExchangeProcessor createMessageExchangeProcessor();
	
	public abstract void doInit() throws JBIException;

	/* (non-Javadoc)
	 * @see javax.jbi.component.ComponentLifeCycle#shutDown()
	 */
	public void shutDown() throws JBIException {
		log.log(Level.INFO, "Shutting down abstract component lifecycle");
		doShutDown();
		processor.shutdown();
		if (mBeanName == null) {
			return ;
		}
		if (mBeanServer == null) {
			log.warning("A null MBeanServer was detected.");
			return ;
		}
		try {
			if (mBeanServer.isRegistered(mBeanName)) {
				mBeanServer.unregisterMBean(mBeanName);
			}
		} catch (Exception e) {
			log.throwing(e.getMessage(), null, e);
			throw new JBIException("Error managing lifecycle MBean: " + e.getMessage(), e);
		}
	}
	
	public abstract void doShutDown() throws JBIException;

	/* (non-Javadoc)
	 * @see javax.jbi.component.ComponentLifeCycle#start()
	 */
	public void start() throws JBIException {
		log.log(Level.INFO, "Starting abstract component lifecycle");
		deliveryChannel = componentContext.getDeliveryChannel();
		processor.start();
		doStart();
	}
	
	public abstract void doStart() throws JBIException;

	/* (non-Javadoc)
	 * @see javax.jbi.component.ComponentLifeCycle#stop()
	 */
	public void stop() throws JBIException {
		log.log(Level.INFO, "Stopping abstract component lifecycle");
		doStop();
		processor.stop();
	}
	
	public abstract void doStop() throws JBIException;

	public DeliveryChannel getDeliveryChannel() {
		return deliveryChannel;
	}

}
