/**
 *   common-jbi-runtime.jar - Set of classes used in building JBI Components
 *
 *   Copyright (C) 2007 Gestalt, LLC. All Rights Reserved.
 *   http://www.gestalt-llc.com/
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License version 2.1 as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 */

package com.gestalt.jbi.component.manager;

import com.gestalt.jbi.component.AbstractComponent;

import javax.xml.namespace.QName;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * @author rriven
 *
 */
public class ServiceUnit {

	private static final Logger log = Logger.getLogger(ServiceUnit.class.getName());
	
	public enum LifeCycle {STARTED, STOPPED, SHUTDOWN};

	private Map<String, Endpoint> endpoints = new HashMap<String, Endpoint>();	
	
	private LifeCycle status = LifeCycle.SHUTDOWN;
	
	private String serviceUnitName;
	private String serviceUnitRootPath;
	
	private AbstractComponent component;
	
	public ServiceUnit(String serviceUnitName, String serviceUnitRootPath, AbstractComponent component) {
		this.serviceUnitName = serviceUnitName;
		this.serviceUnitRootPath = serviceUnitRootPath;
		this.component = component;
	}
	
	public String getServiceUnitName() {
		return serviceUnitName;
	}
	
	public String getServiceUnitRootPath() {
		return serviceUnitRootPath;
	}
	
	public Collection<Endpoint> getEndpoints() {
		return endpoints.values();
	}		
	
	public LifeCycle getStatus() {
		return status;
	}
	
	public AbstractComponent getComponent() {
		return component;
	}
	
	public void start() throws Exception {
		log.log(Level.INFO, "Starting endpoints");
		for (Endpoint endpoint : getEndpoints()) {
			endpoint.activate();
		}
		status = LifeCycle.STARTED;
	}
	
	public void stop() throws Exception {
		log.log(Level.INFO, "Stopping endpoints");
		for (Endpoint endpoint : getEndpoints()) {
			endpoint.deactivate();
		}
		status = LifeCycle.STOPPED;
	}
	
	public void shutdown() throws Exception {
		log.info("Shutting down service unit");
		status = LifeCycle.SHUTDOWN;
	}
	
	public void addEndpoint(Endpoint endpoint) throws Exception {
		log.log (Level.INFO, "Attempting to add endpoint: " + endpoint);
		String endpointKey = generateEndpointKey(endpoint.getServiceName(), endpoint.getEndpointName());
		if (endpoints.put(endpointKey, endpoint) != null) {
			log.log(Level.WARNING, "Endpoint already exists for - " + endpointKey);
			throw new IllegalStateException("Endpoint already exists for - " + endpointKey);
		}
		if (LifeCycle.STARTED.equals(status)) {
			endpoint.activate();
		}
		((AbstractServiceUnitManager) component.getServiceUnitManager()).registerEndpoint(endpointKey, endpoint);
		log.log(Level.INFO ,"Added endpoint");
	}
	
	public static String generateEndpointKey(QName serviceName, String endpointName) {
		StringBuilder builder = new StringBuilder();
		builder.append("[");
		builder.append(serviceName.getNamespaceURI());
		builder.append("]");
		builder.append(serviceName.getLocalPart());
		builder.append(":");
		builder.append(endpointName);
		return builder.toString();
	}

	public void setServiceUnitName(String serviceUnitName) {
		this.serviceUnitName = serviceUnitName;
	}
	
	public void setServiceUnitRootPath(String serviceUnitRootPath) {
		this.serviceUnitRootPath = serviceUnitRootPath;
	}
	
}
